﻿using BookStore.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace BookStore.Controllers
{
    public class WebsiteController : Controller
    {
        // This is the main homepage that lists all of the books
        public ActionResult Home()
        {
            // View model classes like the one below are meant to contain all of the data that the page needs to display.
            HomePageViewModel viewModel = new HomePageViewModel();

            // Entity Framework (from Microsoft) is used heavily in this project. It is a popular framework used by businesses today.
            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                // LINQ is used by Entity Framework to retrieve results.
                viewModel.books = (from b in ctx.AGET_BOOKS_STOCK
                                   select new HomePageViewModel.BookSummary
                                   {
                                        id = b.BOOKID,
                                        title = b.TITLE,
                                        rating = (int) Math.Round((double) b.RATING),
                                        reviewCount = (int) b.REVIEWS,
                                        price = b.PRICE
                                       
                                   }).ToList();

                foreach(HomePageViewModel.BookSummary book in viewModel.books)
                {
                    book.authorNames = (from b in ctx.AGET_BOOK
                                        where b.BOOKID == book.id
                                        from a in b.AGET_AUTHOR
                                        select a.NAME).ToList();
                }

            }

            return View(viewModel);
        }

        // This is the order history page.
        public ActionResult OrderHistory()
        {
            // The logged in user is retrieved from the current session.
            User loggedInUser = HttpContext.Session["UserSession"] as User;
            OrderHistoryViewModel viewModel = new OrderHistoryViewModel();

            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                viewModel.orders = (from o in ctx.AGET_ORDERS_SUMMARY_REPORT
                                    where o.USERID == loggedInUser.id
                                    orderby o.Date descending
                                    select new OrderHistoryViewModel.OrderSummary
                                    {
                                        id = o.ORDERID,
                                        date = o.Date,
                                        totalCost = (decimal) (o.ITEMSCOST + o.SHIPPINGCOST)
                                    }).ToList();
            }

            return View(viewModel);
        }

        // This is the page for viewing a specific order
        public ActionResult OrderView(int id)
        {
            OrderSummaryViewModel viewModel = new OrderSummaryViewModel();

            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                AGET_ORDER order = (from o in ctx.AGET_ORDER
                                    where o.ORDERID == id
                                    select o).FirstOrDefault();

                viewModel.books = (from c in order.AGET_CONTAINS
                                   from b in ctx.AGET_BOOK
                                   where b.BOOKID == c.BOOKID
                                   select new BookOrder(b, c.QUANTITY, order.Date)).ToList();

                viewModel.trackingNumber = (from s in order.AGET_SHIPPEDBY
                                            select s.TRACKINGCODE).FirstOrDefault();

                viewModel.plan = (from s in order.AGET_SHIPPEDBY
                                  select new CarrierPlan
                                  {
                                      id = s.CARRIERPLANID,
                                      name = s.AGET_CARRIERPLAN.NAME,
                                      cost = s.AGET_CARRIERPLAN.COST
                                  }).FirstOrDefault();

                viewModel.address = (from a in ctx.AGET_USERADDRESS
                                     where a.ADDRESSID == order.ADDRESSID
                                     select new Address
                                     {
                                         id = a.ADDRESSID,
                                         address = a.ADDRESS,
                                         city = a.CITY,
                                         apt = a.APT,
                                         state = a.STATE
                                     }).FirstOrDefault();

                viewModel.date = order.Date;
            }

            return View(viewModel);
        }

        // This is the page for the report of all orders.
        public ActionResult OrderReport()
        {
            User loggedInUser = HttpContext.Session["UserSession"] as User;
            OrderReportViewModel viewModel = new OrderReportViewModel();

            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                List<AGET_ORDER> userOrders = (from o in ctx.AGET_ORDER
                                               where o.USERID == loggedInUser.id
                                               orderby o.Date descending
                                               select o).ToList();

                viewModel.books = (from o in userOrders
                                   from c in o.AGET_CONTAINS
                                   from b in ctx.AGET_BOOK
                                   where b.BOOKID == c.BOOKID
                                   select new BookOrder(b, c.QUANTITY, o.Date)).ToList();

            }

            return View(viewModel);
        }

        // This is the logic for submitting a book review, it is called by javascript
        public string SubmitReview(string text, short? rating, int bookID)
        {
            // Make sure we got data
            if (string.IsNullOrEmpty(text))
                return "Invalid data.";

            User loggedInUser = HttpContext.Session["UserSession"] as User;

            AGET_REVIEWS review = new AGET_REVIEWS();
            review.RATING = rating;
            review.BOOKID = bookID;
            review.Text = text;
            review.USERID = loggedInUser.id;
            review.DATESUBMITTED = DateTime.Now;

            // Connect to the database
            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                // The the review and save (insert and commit)
                ctx.AGET_REVIEWS.Add(review);
                ctx.SaveChanges();
            }

            return "Success";
        }

        // This is the logic for submitting an order, it is called by javascript
        public string SubmitOrder(string data)
        {
            // Make sure we got data
            if (string.IsNullOrEmpty(data))
                return "Invalid data.";

            User loggedInUser = HttpContext.Session["UserSession"] as User;

            // Deserialize the data into our order data object.
            OrderData orderInfo = Newtonsoft.Json.JsonConvert.DeserializeObject<OrderData>(data);
            int maxOrder;

            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                int maxAddress = (from a in ctx.AGET_USERADDRESS select a.ADDRESSID).Max() + 1;

                int addressID = orderInfo.existingAddress;
                // If the user did not select an existing address then existingAddress will be -1
                // if that is the case, then it must be inserted.
                if (orderInfo.existingAddress < 0)
                {
                    AGET_USERADDRESS address = new AGET_USERADDRESS();
                    address.ADDRESSID = maxAddress;
                    addressID = address.ADDRESSID;
                    address.ADDRESS = orderInfo.address;
                    address.APT = orderInfo.apt;
                    address.CITY = orderInfo.city;
                    address.STARTDATE = DateTime.Now;
                    address.STATE = orderInfo.state;
                    address.USERID = loggedInUser.id;

                    ctx.AGET_USERADDRESS.Add(address);
                    ctx.SaveChanges();
                }

                maxOrder = (from a in ctx.AGET_ORDER select a.ORDERID).Max() + 1;

                AGET_ORDER order = new AGET_ORDER();
                order.ADDRESSID = addressID;
                order.Date = DateTime.Now;
                order.USERID = loggedInUser.id;
                order.ORDERID = maxOrder;

                ctx.AGET_ORDER.Add(order);
                ctx.SaveChanges();

                // Insert all of the books ordered
                for (int i = 0; i < orderInfo.items.Count(); i++)
                {
                    AGET_CONTAINS contains = new AGET_CONTAINS();
                    contains.BOOKID = orderInfo.items[i].id;
                    contains.QUANTITY = (short)orderInfo.items[i].quantity;
                    contains.ORDERID = maxOrder;

                    ctx.AGET_CONTAINS.Add(contains);
                    ctx.SaveChanges();
                }

                AGET_SHIPPEDBY shippedBy = new AGET_SHIPPEDBY();
                shippedBy.CARRIERPLANID = orderInfo.carrierService;
                shippedBy.ORDERID = maxOrder;

                Random rand = new Random();
                shippedBy.TRACKINGCODE = rand.Next(1000, 9999) + "-" + rand.Next(1000, 9999) + "-" + rand.Next(1000, 9999) + "-" + rand.Next(1000, 9999);

                ctx.AGET_SHIPPEDBY.Add(shippedBy);
                ctx.SaveChanges();
            }

            return maxOrder.ToString();
        }

        // This is the order checkout page.
        public ActionResult Checkout()
        {
            User loggedInUser = HttpContext.Session["UserSession"] as User;
            CheckoutViewModel viewModel = new CheckoutViewModel();

            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                // Select all carriers and their plans.
                viewModel.carriers = (from c in ctx.AGET_SHIPPINGCARRIER.AsEnumerable()
                                      select new Carrier
                                      {
                                          id = c.SHIPPINGCARRIERID,
                                          name = c.NAME,
                                          carrierPlans = (from p in c.AGET_CARRIERPLAN
                                                          select new CarrierPlan
                                                          {
                                                              id = p.CARRIERPLANID,
                                                              name = p.NAME,
                                                              cost = p.COST
                                                          }).ToList()
                                      }).ToList();


                viewModel.existingAddresses = (from a in ctx.AGET_USERADDRESS.AsEnumerable()
                                               where a.USERID == loggedInUser.id
                                               select new Address(a)).ToList();

                viewModel.states = (from st in ctx.AGET_STATES.AsEnumerable()
                                    select new State(st)).ToList();
            }

            return View(viewModel);
        }

        //This is the page for a specific book
        public ActionResult Book(int id = 0)
        {
            using (BookStoreEntities ctx = new BookStoreEntities())
            {
                var results = (from b in ctx.AGET_BOOK
                               where b.BOOKID == id
                               select b);

                AGET_BOOK book = results.FirstOrDefault();
                BookViewModel model = new BookViewModel(book);

                return View(model);
            }
        }
    }
}